@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py 3882 2008-05-07 18:22:10Z richb $"
-__version__   = "$Revision: 3882 $"
-__date__      = "$Date: 2008-05-07 14:22:10 -0400 (Wed, 07 May 2008) $"
+__id__        = "$Id: braillegenerator.py 3957 2008-06-04 16:43:04Z richb $"
+__version__   = "$Revision: 3957 $"
+__date__      = "$Date: 2008-06-04 12:43:04 -0400 (Wed, 04 Jun 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -37,6 +37,7 @@ import rolenames
 import settings
 
 from orca_i18n import _                     # for gettext support
+from orca_i18n import ngettext              # for ngettext support
 
 class BrailleGenerator:
     """Takes accessible objects and produces a list of braille Regions
@@ -121,6 +122,8 @@ class BrailleGenerator:
              self._getBrailleRegionsForRowHeader
         self.brailleGenerators[pyatspi.ROLE_SCROLL_BAR]          = \
              self._getBrailleRegionsForScrollBar
+        self.brailleGenerators[pyatspi.ROLE_SCROLL_PANE]         = \
+             self._getBrailleRegionsForScrollPane
         self.brailleGenerators[pyatspi.ROLE_SLIDER]              = \
              self._getBrailleRegionsForSlider
         self.brailleGenerators[pyatspi.ROLE_SPIN_BUTTON]         = \
@@ -534,7 +537,38 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForFrame", obj)
 
-        return self._getDefaultBrailleRegions(obj)
+        regions = []
+
+        text = ""
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedText(obj))
+        text = self._script.appendString(text,
+                                         self._script.getTextForValue(obj))
+        text = self._script.appendString(text, self._getTextForRole(obj))
+
+        # If this application has more than one unfocused alert or
+        # dialog window, then add '(<m> dialogs)' to the braille context,
+        # to let the user know.
+        #
+        alertAndDialogCount = \
+                    self._script.getUnfocusedAlertAndDialogCount(obj)
+        if alertAndDialogCount > 0:
+            # Translators: this tells the user how many unfocused
+            # alert and dialog windows plus the total number of
+            # windows that this application has.
+            #
+            line = ngettext("(%d dialog)",
+                            "(%d dialogs)",
+                            alertAndDialogCount) % alertAndDialogCount
+            text = self._script.appendString(text, line)
+
+        regions = []
+        componentRegion = braille.Component(obj, text)
+        regions.append(componentRegion)
+
+        return [regions, componentRegion]
 
     def _getBrailleRegionsForHtmlContainer(self, obj):
         """Get the braille for an HTML container.
@@ -1060,6 +1094,30 @@ class BrailleGenerator:
 
         return self._getDefaultBrailleRegions(obj)
 
+    def _getBrailleRegionsForScrollPane(self, obj):
+        """Get the braille for a scroll pane.
+
+        Arguments:
+        - obj: the scroll pane
+
+        Returns a list where the first element is a list of Regions to display
+        and the second element is the Region which should get focus.
+        """
+
+        self._debugGenerator("_getBrailleRegionsForScrollPane", obj)
+
+        # If this scroll pane is labelled by a page tab, then return the
+        # page tab information for the braille context instead. Thunderbird
+        # folder properties is such a case. See bug #507922 for more details.
+        #
+        relations = obj.getRelationSet()
+        for relation in relations:
+            if relation.getRelationType() ==  pyatspi.RELATION_LABELLED_BY:
+                labelledBy = relation.getTarget(0)
+                return self._getBrailleRegionsForPageTab(labelledBy)
+
+        return self._getDefaultBrailleRegions(obj)
+
     def _getBrailleRegionsForSlider(self, obj):
         """Get the braille for a slider.  If the object already
         had focus, just the value is displayed.
@@ -1222,7 +1280,8 @@ class BrailleGenerator:
 
                     if label == None or len(label) == 0:
                         table = obj.parent.queryTable()
-                        n = table.getColumnAtIndex(obj.getIndexInParent())
+                        index = self._script.getCellIndex(obj)
+                        n = table.getColumnAtIndex(index)
                         accHeader = table.getColumnHeader(n)
                         regions[0].append(braille.Region(" "))
                         label = accHeader.name
@@ -1308,9 +1367,9 @@ class BrailleGenerator:
             savedBrailleVerbosityLevel = settings.brailleVerbosityLevel
             settings.brailleVerbosityLevel = \
                                          settings.VERBOSITY_LEVEL_BRIEF
-
-            row = table.getRowAtIndex(obj.getIndexInParent())
-            column = table.getColumnAtIndex(obj.getIndexInParent())
+            index = self._script.getCellIndex(obj)
+            row = table.getRowAtIndex(index)
+            column = table.getColumnAtIndex(index)
 
             # This is an indication of whether we should speak all the
             # table cells (the user has moved focus up or down a row),
@@ -1318,8 +1377,8 @@ class BrailleGenerator:
             # the same row).
             #
             speakAll = True
-            if self._script.pointOfReference.has_key("lastRow") and \
-                self._script.pointOfReference.has_key("lastColumn"):
+            if "lastRow" in self._script.pointOfReference and \
+                "lastColumn" in self._script.pointOfReference:
                 pointOfReference = self._script.pointOfReference
                 speakAll = (pointOfReference["lastRow"] != row) or \
                        ((row == 0 or row == table.nRows-1) and \
@@ -1537,7 +1596,7 @@ class BrailleGenerator:
                     or (obj.getRole() == pyatspi.ROLE_PAGE_TAB_LIST)
 
         role = obj.getRole()
-        if self.brailleGenerators.has_key(role):
+        if role in self.brailleGenerators:
             generator = self.brailleGenerators[role]
         else:
             generator = self._getDefaultBrailleRegions
@@ -1655,7 +1714,8 @@ class BrailleGenerator:
         except (NotImplementedError, AttributeError):
             table = None
         if parent and table:
-            row = table.getRowAtIndex(obj.getIndexInParent())
+            index = self._script.getCellIndex(obj)
+            row = table.getRowAtIndex(index)
             if (row >= 0) \
                 and (not obj.getRole() in [pyatspi.ROLE_ROW_HEADER,
                                            pyatspi.ROLE_TABLE_ROW_HEADER]):
@@ -1676,7 +1736,7 @@ class BrailleGenerator:
                     text = desc
                 regions.append(braille.Region(text))
 
-            col = table.getColumnAtIndex(obj.getIndexInParent())
+            col = table.getColumnAtIndex(index)
             if (col >= 0) \
                 and (not obj.getRole() in [pyatspi.ROLE_COLUMN_HEADER,
                                            pyatspi.ROLE_TABLE_COLUMN_HEADER]):
