@@ -19,9 +19,9 @@
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__  = "$Id: focus_tracking_presenter.py 3882 2008-05-07 18:22:10Z richb $"
-__version__   = "$Revision: 3882 $"
-__date__      = "$Date: 2008-05-07 14:22:10 -0400 (Wed, 07 May 2008) $"
+__id__  = "$Id: focus_tracking_presenter.py 3957 2008-06-04 16:43:04Z richb $"
+__version__   = "$Revision: 3957 $"
+__date__      = "$Date: 2008-06-04 12:43:04 -0400 (Wed, 04 Jun 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -102,7 +102,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         - eventType: the event type.
         """
 
-        if self._listenerCounts.has_key(eventType):
+        if eventType in self._listenerCounts:
             self._listenerCounts[eventType] += 1
         else:
             self.registry.registerEventListener(self._enqueueEvent,
@@ -289,7 +289,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 self._defaultScript = default.Script(None)
                 self._registerEventListeners(self._defaultScript)
             script = self._defaultScript
-        elif self._knownScripts.has_key(app):
+        elif app in self._knownScripts:
             script = self._knownScripts[app]
         else:
             script = self._createScript(app)
@@ -404,7 +404,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 try:
                     debug.println(debug.LEVEL_FINEST,
                                   "Looking for settings at %s.py..." % name)
-                    if not self._knownAppSettings.has_key(name):
+                    if name not in self._knownAppSettings:
                         self._knownAppSettings[name] = \
                             __import__(name, globals(), locals(), [''])
                     reload(self._knownAppSettings[name])
@@ -605,9 +605,18 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     # in use, and then issues a focus: event on the
                     # main window, which is a frame.]]]
                     #
-                    if (event.type == "window:activate") \
-                       or ((event.type.startswith("focus")) 
-                           and (event.source.getRole() == pyatspi.ROLE_FRAME)):
+                    # Added in a further check. We look for modal panels
+                    # that are now showing (such as gnome-screensaver-dialog).
+                    # See bug #530368 for more details.
+                    #
+                    eType = event.type
+                    if (eType == "window:activate") \
+                       or ((eType.startswith("focus")) 
+                         and (event.source.getRole() == pyatspi.ROLE_FRAME)) \
+                       or (eType.startswith("object:state-changed:showing")
+                         and (event.source.getRole() == pyatspi.ROLE_PANEL)
+                         and state.contains(pyatspi.STATE_MODAL)):
+
                         # We'll let someone else decide if it's important
                         # to stop speech or not.
                         #speech.stop()
@@ -663,6 +672,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         - e: an at-spi event.
         """
 
+        asyncMode = settings.asyncMode
+
         if settings.debugEventQueue:
             if self._enqueueEventCount:
                 debug.println(debug.LEVEL_ALL,
@@ -735,6 +746,16 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 debug.println(debug.LEVEL_ALL,
                               "---------> QUEUEING EVENT %s" % e.type)
 
+            # Some toolkits (e.g., Java - see bug #531869) need to
+            # have their events processed immediately.
+            #
+            try:
+                if event.host_application.toolkitName \
+                    in settings.synchronousToolkits:
+                    asyncMode = False
+            except:
+                pass
+
         if event:
             if settings.debugEventQueue:
                 debug.println(debug.LEVEL_ALL,
@@ -751,7 +772,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             if settings.debugEventQueue:
                 debug.println(debug.LEVEL_ALL,
                               "           ...put complete")
-            if settings.asyncMode and (not self._gidleId):
+            if asyncMode and (not self._gidleId):
                 if settings.gilSleepTime:
                     time.sleep(settings.gilSleepTime)
                 self._gidleId = gobject.idle_add(self._dequeueEvent)
@@ -764,7 +785,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 debug.println(debug.LEVEL_ALL,
                               "           ...released")
 
-            if not settings.asyncMode:
+            if not asyncMode:
                 self._dequeueEvent()
 
         if settings.debugEventQueue:
