@@ -19,9 +19,9 @@
 
 """Provides a SpeechServer factory for gnome-speech drivers."""
 
-__id__        = "$Id: gnomespeechfactory.py 3882 2008-05-07 18:22:10Z richb $"
-__version__   = "$Revision: 3882 $"
-__date__      = "$Date: 2008-05-07 14:22:10 -0400 (Wed, 07 May 2008) $"
+__id__        = "$Id: gnomespeechfactory.py 3978 2008-06-16 16:30:59Z wwalker $"
+__version__   = "$Revision: 3978 $"
+__date__      = "$Date: 2008-06-16 12:30:59 -0400 (Mon, 16 Jun 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -52,6 +52,50 @@ from orca_i18n import _           # for gettext support
 ORBit.load_typelib('GNOME_Speech')
 import GNOME.Speech, GNOME__POA.Speech
 
+def _matchLanguage(desired, actual):
+    """Compares a desired language to an actual language and returns
+    a numerical value representing the degree of match.  The expected
+    language format is a string similar to the following, where the
+    delimiter can be '-' '_' or '.':
+
+      de-DE (German for Germany)
+      en-US (English as used in the United States)
+
+    Additional extensions for variants or dialects are allowed as well.
+
+    The return value is one of the following:
+
+      0 - no match at all
+      1 - some match
+      2 - exact match
+    """
+
+    if desired == actual:
+        return 2
+    if not desired or not actual:
+        return 0
+
+    # Break the strings into arrays where the delimiters are
+    # - or _.  Also lowercase everything to help with string 
+    # compares.
+    #
+    desired = desired.lower().replace("-","_").replace(".","_").split("_")
+    actual = actual.lower().replace("-","_").replace(".","_").split("_")
+
+    matchCount = 0
+    for i in range(0, min(len(desired), len(actual))):
+        if desired[i] == actual[i]:
+            matchCount += 1
+        else:
+            break
+
+    if matchCount == len(desired):
+        return 2
+    elif matchCount:
+        return 1
+    else:
+        return 0
+        
 class _SayAll:
     def __init__(self, iterator, context, utteranceId, progressCallback):
         self.utteranceIterator   = iterator
@@ -160,7 +204,7 @@ class SpeechServer(speechserver.SpeechServer):
     def __createServer(iid):
         server = None
 
-        if SpeechServer.__activeServers.has_key(iid):
+        if iid in SpeechServer.__activeServers:
             server = SpeechServer.__activeServers[iid]
         else:
             driver = SpeechServer.__activateDriver(iid)
@@ -191,7 +235,7 @@ class SpeechServer(speechserver.SpeechServer):
             "repo_ids.has('IDL:GNOME/Speech/SynthesisDriver:0.3')")
 
         for server in knownServers:
-            if not SpeechServer.__activeServers.has_key(server.iid):
+            if server.iid not in SpeechServer.__activeServers:
                 try:
                     SpeechServer.__createServer(server.iid)
                 except:
@@ -206,7 +250,7 @@ class SpeechServer(speechserver.SpeechServer):
         See SpeechServer.getInfo() for more info.
         """
 
-        if info and SpeechServer.__activeServers.has_key(info[1]):
+        if info and info[1] in SpeechServer.__activeServers:
             return SpeechServer.__activeServers[info[1]]
 
         server = None
@@ -283,7 +327,7 @@ class SpeechServer(speechserver.SpeechServer):
     def __getRate(self, speaker):
         """Gets the voice-independent ACSS rate value of a voice."""
 
-        if not self.__rateInfo.has_key(speaker):
+        if speaker not in self.__rateInfo:
             return 50.0
 
         [minRate, averageRate, maxRate] = self.__rateInfo[speaker]
@@ -301,7 +345,7 @@ class SpeechServer(speechserver.SpeechServer):
         voice-independent ACSS rate value.
         """
 
-        if not self.__rateInfo.has_key(speaker):
+        if speaker not in self.__rateInfo:
             return
 
         [minRate, averageRate, maxRate] = self.__rateInfo[speaker]
@@ -322,7 +366,7 @@ class SpeechServer(speechserver.SpeechServer):
         Returns the voice-specific pitch setting.
         """
 
-        if not self.__pitchInfo.has_key(speaker):
+        if speaker not in self.__pitchInfo:
             return 5.0
 
         [minPitch, averagePitch, maxPitch] = self.__pitchInfo[speaker]
@@ -340,7 +384,7 @@ class SpeechServer(speechserver.SpeechServer):
         voice-independent ACSS pitch value.
         """
 
-        if not self.__pitchInfo.has_key(speaker):
+        if speaker not in self.__pitchInfo:
             return
 
         [minPitch, averagePitch, maxPitch] = self.__pitchInfo[speaker]
@@ -359,7 +403,7 @@ class SpeechServer(speechserver.SpeechServer):
         voice-independent ACSS rate value.
         """
 
-        if not self.__volumeInfo.has_key(speaker):
+        if speaker not in self.__volumeInfo:
             return
 
         [minVolume, averageVolume, maxVolume] = self.__volumeInfo[speaker]
@@ -375,7 +419,7 @@ class SpeechServer(speechserver.SpeechServer):
         if not acss:
             acss = defaultACSS
 
-        if self.__speakers.has_key(acss.name()):
+        if acss.name() in self.__speakers:
             return self.__speakers[acss.name()]
 
         # Search for matching languages first, as that is the most
@@ -384,10 +428,10 @@ class SpeechServer(speechserver.SpeechServer):
         #
         languages = []
         try:
-            if acss.has_key(ACSS.FAMILY):
+            if ACSS.FAMILY in acss:
                 family = acss[ACSS.FAMILY]
                 languages = [family[speechserver.VoiceFamily.LOCALE]]
-            elif defaultACSS.has_key(ACSS.FAMILY):
+            elif ACSS.FAMILY in defaultACSS:
                 family = defaultACSS[ACSS.FAMILY]
                 languages = [family[speechserver.VoiceFamily.LOCALE]]
         except:
@@ -411,7 +455,16 @@ class SpeechServer(speechserver.SpeechServer):
         voices = self.__driver.getAllVoices()
         foundVoices = []
         for voice in voices:
-            if voice.language in languages:
+            match = 0
+            for language in languages:
+                match = max(match, _matchLanguage(voice.language, language))
+
+            # Exact matches get put at the beginning, inexact get added to
+            # the end.
+            #
+            if match == 2:
+                foundVoices.insert(0, voice)
+            elif match:
                 foundVoices.append(voice)
 
         # If we didn't find any matches, well...punt.
@@ -424,10 +477,10 @@ class SpeechServer(speechserver.SpeechServer):
         # Now search for a matching family name.
         #
         familyName = None
-        if acss.has_key(ACSS.FAMILY):
+        if ACSS.FAMILY in acss:
             family = acss[ACSS.FAMILY]
             familyName = family[speechserver.VoiceFamily.NAME]
-        elif defaultACSS.has_key(ACSS.FAMILY):
+        elif ACSS.FAMILY in defaultACSS:
             family = defaultACSS[ACSS.FAMILY]
             familyName = family[speechserver.VoiceFamily.NAME]
 
@@ -454,7 +507,7 @@ class SpeechServer(speechserver.SpeechServer):
         speaker = _Speaker(s._narrow(GNOME.Speech.Speaker))
 
         # Turn off punctuation.  We do this because we want to handle
-        # spoken punctuation on our own.  Only do so if "punctuation mode" 
+        # spoken punctuation on our own.  Only do so if "punctuation mode"
         # is available (see # bug #528797).
         #
         try:
@@ -480,13 +533,13 @@ class SpeechServer(speechserver.SpeechServer):
                 self.__volumeInfo[speaker] = \
                     [parameter.min, parameter.current, parameter.max]
 
-        if acss.has_key(ACSS.RATE):
+        if ACSS.RATE in acss:
             self.__setRate(speaker, acss[ACSS.RATE])
 
-        if acss.has_key(ACSS.AVERAGE_PITCH):
+        if ACSS.AVERAGE_PITCH in acss:
             self.__setPitch(speaker, acss[ACSS.AVERAGE_PITCH])
 
-        if acss.has_key(ACSS.GAIN):
+        if ACSS.GAIN in acss:
             self.__setVolume(speaker, acss[ACSS.GAIN])
 
         self.__speakers[acss.name()] = speaker
@@ -652,7 +705,7 @@ class SpeechServer(speechserver.SpeechServer):
                      used to augment/override the default
                      voice settings.
         """
-        self.speak(character, acss)
+        self.speak(chnames.getCharacterName(character), acss)
 
     def speakUtterances(self, utterances, acss=None, interrupt=True):
         """Speaks the given list of utterances immediately.
@@ -722,7 +775,7 @@ class SpeechServer(speechserver.SpeechServer):
         ##
         removeNewLines = True
         if orca_state.lastInputEvent and \
-               orca_state.lastInputEvent.__dict__.has_key("event_string"):
+               "event_string" in orca_state.lastInputEvent.__dict__:
             lastKey = orca_state.lastInputEvent.event_string
             if lastKey == "Left" or lastKey == "Right":
                 removeNewLines = False
@@ -829,10 +882,10 @@ class SpeechServer(speechserver.SpeechServer):
             return -1
 
         speaker = self.__getSpeaker(acss)
-        if acss and not acss.has_key(ACSS.RATE):
+        if acss and ACSS.RATE not in acss:
             voices = settings.voices
             defaultACSS = voices[settings.DEFAULT_VOICE]
-            if defaultACSS.has_key(ACSS.RATE):
+            if ACSS.RATE in defaultACSS:
                 self.__setRate(speaker, defaultACSS[ACSS.RATE])
 
         if not text:
@@ -881,6 +934,13 @@ class SpeechServer(speechserver.SpeechServer):
         if self.__sayAll:
             self.stop()
 
+        if not acss and text and text.isupper():
+            try:
+                acss = \
+                    orca_state.activeScript.voices[settings.UPPERCASE_VOICE]
+            except:
+                pass
+
         self.__speak(text, acss, interrupt)
 
     def isSpeaking(self):
@@ -1049,7 +1109,7 @@ class SpeechServer(speechserver.SpeechServer):
 
     def shutdown(self):
         """Shuts down the speech engine."""
-        if SpeechServer.__activeServers.has_key(self.__iid):
+        if self.__iid in SpeechServer.__activeServers:
             for speaker in self.__speakers.values():
                 speaker.stop()
                 speaker.unref()
