@@ -14,14 +14,14 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py 3819 2008-04-07 19:20:05Z wwalker $"
-__version__   = "$Revision: 3819 $"
-__date__      = "$Date: 2008-04-07 15:20:05 -0400 (Mon, 07 Apr 2008) $"
+__id__        = "$Id: orca.py 3975 2008-06-12 19:08:52Z wwalker $"
+__version__   = "$Revision: 3975 $"
+__date__      = "$Date: 2008-06-12 15:08:52 -0400 (Thu, 12 Jun 2008) $"
 __copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -41,10 +41,15 @@ try:
     # the desktop is not running.
     #
     import gtk
-    import mouse_review
 except:
     pass
 
+try:
+    # If we don't have an active desktop, we will get a RuntimeError.
+    import mouse_review
+except RuntimeError:
+    pass
+
 import getopt
 import gnome
 import os
@@ -71,7 +76,6 @@ from input_event import MouseButtonEvent
 from input_event import keyEventToString
 
 from orca_i18n import _           # for gettext support
-from orca_i18n import Q_          # to provide qualified translatable strings
 
 if settings.debugMemoryUsage:
     import gc
@@ -152,7 +156,12 @@ def getScriptForApp(app):
 
     Returns a Script instance.
     """
-    return _PRESENTATION_MANAGERS[_currentPresentationManager].getScript(app)
+
+    script = None
+    if _currentPresentationManager >= 0:
+        script = \
+            _PRESENTATION_MANAGERS[_currentPresentationManager].getScript(app)
+    return script
 
 ########################################################################
 #                                                                      #
@@ -173,6 +182,15 @@ def setLocusOfFocus(event, obj, notifyPresentationManager=True):
     if obj == orca_state.locusOfFocus:
         return
 
+    # If this event is not for the currently active script, then just return.
+    #
+    if event and event.source and \
+       event.host_application and orca_state.activeScript:
+        currentApp = orca_state.activeScript.app
+        if currentApp != event.host_application and \
+           currentApp != event.source.getApplication():
+            return
+
     oldLocusOfFocus = orca_state.locusOfFocus
     try:
         # Just to see if we have a valid object.
@@ -310,6 +328,27 @@ def cycleDebugLevel(script=None, inputEvent=None):
 
     return True
 
+def exitLearnMode(self, inputEvent=None):
+    """Turns learn mode off.
+
+    Returns True to indicate the input event has been consumed.
+    """
+
+    # Translators: Orca has a "Learn Mode" that will allow
+    # the user to type any key on the keyboard and hear what
+    # the effects of that key would be.  The effects might
+    # be what Orca would do if it had a handler for the
+    # particular key combination, or they might just be to
+    # echo the name of the key if Orca doesn't have a handler.
+    # Exiting learn mode puts the user back in normal operating
+    # mode.
+    #
+    message = _("Exiting learn mode.")
+    speech.speak(message)
+    braille.displayMessage(message)
+    settings.learnModeEnabled = False
+    return True
+
 ########################################################################
 #                                                                      #
 # METHODS FOR PRE-PROCESSING AND MASSAGING KEYBOARD EVENTS.            #
@@ -653,9 +692,9 @@ def _processKeyboardEvent(event):
 
     # See if this is one of our special Orca modifier keys.
     #
-    # Just looking at the keycode should suffice, but there is a 
-    # "feature" in the Java Access Bridge where it chooses to emit 
-    # Java platform-independent keycodes instead of the keycodes 
+    # Just looking at the keycode should suffice, but there is a
+    # "feature" in the Java Access Bridge where it chooses to emit
+    # Java platform-independent keycodes instead of the keycodes
     # for the base platform:
     #
     # http://bugzilla.gnome.org/show_bug.cgi?id=106004
@@ -708,7 +747,7 @@ def _processKeyboardEvent(event):
 
     if _orcaModifierPressed:
         keyboardEvent.modifiers = keyboardEvent.modifiers \
-                                  | (1 << settings.MODIFIER_ORCA)
+                                  | settings.ORCA_MODIFIER_MASK
 
     orca_state.lastInputEvent = keyboardEvent
 
@@ -752,6 +791,10 @@ def _processKeyboardEvent(event):
                         event_string = keyboardEvent.event_string
                         event_string = keynames.getKeyName(event_string)
                         speech.speak(event_string)
+                elif (event.type == pyatspi.KEY_RELEASED_EVENT) and \
+                     (keyboardEvent.event_string == "Escape"):
+                    exitLearnMode(keyboardEvent)
+
                 consumed = True
     except:
         debug.printException(debug.LEVEL_SEVERE)
@@ -904,7 +947,6 @@ def loadUserSettings(script=None, inputEvent=None):
         debug.println(debug.LEVEL_CONFIGURATION,
                       "Magnification module has NOT been initialized.")
 
-
     # I'm not sure where else this should go. But it doesn't really look
     # right here.
     try:
@@ -1030,7 +1072,7 @@ def helpForOrca(script=None, inputEvent=None):
 
     props = { gnome.PARAM_APP_DATADIR : '/usr/share'}
     prog = gnome.program_init('orca', '1.0', properties=props)
-    gnome.help_display_with_doc_id(prog, 'gnome-access-guide', 
+    gnome.help_display_with_doc_id(prog, 'gnome-access-guide',
                                          'gnome-access-guide', 'ats-2')
 
     return True
@@ -1155,7 +1197,7 @@ def start(registry):
 
     if settings.timeoutCallback and (settings.timeoutTime > 0):
         signal.alarm(0)
-    
+
     if settings.cacheValues:
         pyatspi.setCacheLevel(pyatspi.CACHE_PROPERTIES)
 
@@ -1283,7 +1325,7 @@ def usage():
     print _("Usage: orca [OPTION...]")
     print
 
-    # Translators: this is the description of the command line option 
+    # Translators: this is the description of the command line option
     # '-?, --help' that is used to display usage information.
     #
     print "-?, --help                   " + _("Show this help message")
@@ -1291,22 +1333,22 @@ def usage():
     print "-v, --version                %s" % platform.version
 
     # Translators: this is the description of the command line option
-    # '-s, --setup, --gui-setup' that will initially display a GUI dialog 
+    # '-s, --setup, --gui-setup' that will initially display a GUI dialog
     # that would allow the user to set their Orca preferences.
     #
     print "-s, --setup, --gui-setup     " + _("Set up user preferences")
 
-    # Translators: this is the description of the command line option 
-    # '-t, --text-setup' that will initially display a list of questions 
-    # in text form, that the user will need to answer, before Orca will 
-    # startup. For this to happen properly, Orca will need to be run 
+    # Translators: this is the description of the command line option
+    # '-t, --text-setup' that will initially display a list of questions
+    # in text form, that the user will need to answer, before Orca will
+    # startup. For this to happen properly, Orca will need to be run
     # from a terminal window.
     #
     print "-t, --text-setup             " + \
           _("Set up user preferences (text version)")
 
-    # Translators: this is the description of the command line option 
-    # '-n, --no-setup' that means that Orca will startup without setting 
+    # Translators: this is the description of the command line option
+    # '-n, --no-setup' that means that Orca will startup without setting
     # up any user preferences.
     #
     print "-n, --no-setup               " +  \
@@ -1314,7 +1356,7 @@ def usage():
 
     # Translators: by default, Orca expects to find its user preferences
     # in a directory called .orca under the user's home directory. This
-    # is the description of the command line option 
+    # is the description of the command line option
     # '-u, --user-prefs-dir=dirname' that allows you to specify an alternate
     # location for those user preferences.
     #
@@ -1322,11 +1364,11 @@ def usage():
           _("Use alternate directory for user preferences")
 
     print "-e, --enable=[" \
-        + Q_("option|speech") + "|" \
-        + Q_("option|braille") + "|" \
-        + Q_("option|braille-monitor") + "|" \
-        + Q_("option|magnifier") + "|" \
-        + Q_("option|main-window") + "]",
+        + "speech" + "|" \
+        + "braille" + "|" \
+        + "braille-monitor" + "|" \
+        + "magnifier" + "|" \
+        + "main-window" + "]",
 
     # Translators: if the user supplies an option via the '-e, --enable'
     # command line option, it will be automatically enabled as Orca is
@@ -1335,17 +1377,17 @@ def usage():
     print _("Force use of option")
 
     print "-d, --disable=[" \
-        + Q_("option|speech") + "|" \
-        + Q_("option|braille") + "|" \
-        + Q_("option|braille-monitor") + "|" \
-        + Q_("option|magnifier") + "|" \
-        + Q_("option|main-window") + "]",
+        + "speech" + "|" \
+        + "braille" + "|" \
+        + "braille-monitor" + "|" \
+        + "magnifier" + "|" \
+        + "main-window" + "]",
 
     # Translators: if the user supplies an option via the '-d, --disable'
     # command line option, it will be automatically disabled as Orca is
     # started.
     #
-    print _("Prevent use of option") 
+    print _("Prevent use of option")
 
     # Translators: this is the Orca command line option that will quit Orca.
     # The user would run the Orca shell script again from a terminal window.
@@ -1357,13 +1399,21 @@ def usage():
 
     print
 
-    # Translators: this is text being sent to a terminal and we want to 
+    # Translators: this is text being sent to a terminal and we want to
     # keep the text lines within terminal boundaries.
     #
     print _("If Orca has not been previously set up by the user, Orca\n" \
             "will automatically launch the preferences set up unless\n" \
             "the -n or --no-setup option is used.")
 
+    # Translators: this is more text being sent to a terminal and we want to
+    # keep the text lines within terminal boundaries.
+    #
+    print
+    print _("WARNING: suspending Orca, e.g. by pressing Control-Z, from\n" \
+            "an AT-SPI enabled shell (such as gnome-terminal), can also\n" \
+            "suspend the desktop until Orca is killed.")
+
     print
     print _("Report bugs to orca-list@gnome.org.")
 
@@ -1451,61 +1501,31 @@ def main():
             if opt in ("-e", "--enable"):
                 feature = val.strip()
 
-                # Translators: this option is for enabling speech synthesis
-                # output.
-                #
-                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-                #
-                if feature == Q_("option|speech"):
+                if feature == "speech":
                     _commandLineSettings["enableSpeech"] = True
-
-                # Translators: this option is for enabling braille output on
-                # a physical refreshable braille display.
-                #
-                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-                #
-                elif feature == Q_("option|braille"):
+                elif feature == "braille":
                     _commandLineSettings["enableBraille"] = True
-
-                # Translators: this option is for enabling a GUI to monitor 
-                # what is being sent to the physical braille display.
-                #
-                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-                #
-                elif feature == Q_("option|braille-monitor"):
+                elif feature == "braille-monitor":
                     _commandLineSettings["enableBrailleMonitor"] = True
-
-                # Translators: this option is for enabling screen 
-                # magnification.
-                #
-                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-                #
-                elif feature == Q_("option|magnifier"):
+                elif feature == "magnifier":
                     _commandLineSettings["enableMagnifier"] = True
-
-                # Translators: this option is for enabling the display
-                # of the Orca main window.
-                #
-                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-                #
-                elif feature == Q_("option|main-window"):
+                elif feature == "main-window":
                     _commandLineSettings["showMainWindow"] = True
-
                 else:
                     usage()
                     abort(2)
 
             if opt in ("-d", "--disable"):
                 feature = val.strip()
-                if feature == Q_("option|speech"):
+                if feature == "speech":
                     _commandLineSettings["enableSpeech"] = False
-                elif feature == Q_("option|braille"):
+                elif feature == "braille":
                     _commandLineSettings["enableBraille"] = False
-                elif feature == Q_("option|braille-monitor"):
+                elif feature == "braille-monitor":
                     _commandLineSettings["enableBrailleMonitor"] = False
-                elif feature == Q_("option|magnifier"):
+                elif feature == "magnifier":
                     _commandLineSettings["enableMagnifier"] = False
-                elif feature == Q_("option|main-window"):
+                elif feature == "main-window":
                     _commandLineSettings["showMainWindow"] = False
                 else:
                     usage()
